#procedure AlgC(i)
****************************************************************************
**
**  Filename:
**            AlgC.prc
**
**  Authors: 
**            Sven O. Moch and Peter Uwer
**
**  Purpose: 
**            Implementation of the conjugation algorithm (type C) to perform sums 
**            of the form
**            \begin{eqnarray*}
**               \sum\limits_{i=1}^{n}\, 
**                 \left( \begin{array}{c} n \\ i \end{array} \right)\, 
**                   (-1)^i {x_1^i \over i^{m_1}} 
**                   S(i;m_2,...,m_k;x_2,...,x_k)\, 
**            \end{eqnarray*}
**            for any (integer) values of $m_i$ and $m_1>0$. 
**            The argument specifies 
**            the index $i$ of the sum to be done. 
**            The special case $m_1=0$ is treated in \keyword{AlgC0}.
**
**  Comments:
**
**
**  Revision: 
**            $Id: AlgC.prc,v 1.0 2005/08/02 10:09:54 puwer Exp $
**
**            $Log: AlgC.prc,v $
**            Revision 1.0  2005/08/02 10:09:54  puwer
**            Initial revision
**
**
****************************************************************************
*
* Convert the nested sums
*
id S`i'(?iii,0,j`i') = SAlgCS`i'(RR,XX,?iii);

repeat, id SAlgCS`i'(RR(?m3),XX(?x3),R(m1?,m2?,?m),X(x1?,x2?,?x)) = 
  + SAlgCS`i'(RR(?m3),XX(?x3),R(m1,m2,?m),X(x1,x2,?x),inf)
  - SAlgCS`i'(RR(?m3,m1),XX(?x3,x1),R(m2,?m),X(x2,?x),inf)
  + SAlgCS`i'(RR(?m3,m1,m2),XX(?x3,x1,x2),R(?m),X(?x)) ;

id SAlgCS`i'(RR(?m3),XX(?x3),R(m1?),X(x1?))
  = SAlgCS`i'(RR(?m3),XX(?x3),R(m1),X(x1),inf)
  - SAlgCS`i'(RR(?m3,m1),XX(?x3,x1));

id SAlgCS`i'(RR(?m1),XX(?x1),R(?m2),X(?x2),inf)
  = SAlgCS`i'(RR(?m1),XX(?x1)) * S(R(?m2),X(?x2),inf);

id SAlgCS`i'(RR(?m1),XX(?x1),R,X) = SAlgCS`i'(RR(?m1),XX(?x1));
id SAlgCS`i'(RR,XX) = 1;

*
* Use one function SAlgCS`i' to collect everything which
* can be solved in this procedure
*
id sum`i'(j`i',n1?{,1},n?) * bino`i'(n?,0,j`i') * pow`i'(x?,j`i') * 
   den`i'(m?,0,j`i') * SAlgCS`i'(RR(?m1),XX(?x1)) = 
    - SAlgCS`i'(R,X,RR(m,?m1),XX(-x,?x1),n);
id sum`i'(j`i',n1?{,0},n?) * bino`i'(n?,0,j`i') * pow`i'(x?,j`i') * 
   den`i'(m?{,0},0,j`i') * SAlgCS`i'(RR(?m1),XX(?x1)) = 
    - SAlgCS`i'(R,X,RR(m,?m1),XX(-x,?x1),n);

id sum`i'(j`i',n1?{,1},n?) * bino`i'(n?,0,j`i') * pow`i'(x?,j`i') * 
   den`i'(m?,0,j`i') = - SAlgCS`i'(R,X,RR(m),XX(-x),n);
id sum`i'(j`i',n1?{,0},n?) * bino`i'(n?,0,j`i') * pow`i'(x?,j`i') * 
   den`i'(m?{,0},0,j`i') = - SAlgCS`i'(R,X,RR(m),XX(-x),n);

repeat;
*
* first reduce m_0 to zero:
*
  repeat,id SAlgCS`i'(R(?m2),X(?x2),RR(m0?{>0},?m1),XX(?x1),n?)
     = SAlgCS`i'(R(?m2,1),X(?x2,1),RR(m0-1,?m1),XX(?x1),n) ;

* 
* now reduce the next index by one to reproduce the startingpoint
* of the iteration:
* x0 = 1 should be treated first
*
  id SAlgCS`i'(R(?m0,m3?),X(?x2),RR(0,m1?,?m2),XX(1,?x1),n?) =
    - SAlgCS`i'(R(?m0,m3+1),X(?x2),RR(m1-1,?m2),XX(?x1),n)
    + S(R(?m0,m3),X(?x2),n) * SAlgCSS(RR(m1,?m2),XX(?x1));
*
* now the general case x0 \not = 1:
*
  id SAlgCS`i'(R(?m0),X(?x2,x3?),RR(0,m1?,?m2),XX(x0?!{,1},x4?,?x1),n?) =
    - SAlgCS`i'(R(?m0,1),X(?x2,x3*num(1-x0),den(1-x0)),
            RR(m1-1,?m2),XX(x0*x4,?x1),n)
    + S(R(?m0),X(?x2,x3),n) * SAlgCSS(RR(m1,?m2),XX(x4,?x1))
    - S(R(?m0),X(?x2,x3*num(1-x0)),n) * SAlgCSS(RR(m1,?m2),XX(x4,?x1));
*
* if m_0 was zero from the begining we have:
*
* for x0 = 1:
*
  id SAlgCS`i'(R,X,RR(0,m1?,?m2),XX(x0?{,1},x4?,?x1),n?) =
    - den(n) * SAlgCS`i'(R,X,RR(m1-1,?m2),XX(x4,?x1),n)
    + SAlgCSS(RR(m1,?m2),XX(x4,?x1));
*
* and now for x0\not = 1
*
  id SAlgCS`i'(R,X,RR(0,m1?,?m2),XX(x0?!{,1},x4?,?x1),n?) =
    - pow(1-x0,n) * SAlgCS`i'(R(1),X(den(1-x0)),RR(m1-1,?m2),XX(x0*x4,?x1),n)
    + ( 1 - pow(1-x0,n) ) * SAlgCSS(RR(m1,?m2),XX(x4,?x1));

#call SimpleArgs(C)
 
endrepeat;

*
* go back to standard notation:
*
id SAlgCS`i'(R(?m2),X(?x2,x3?),RR(0),XX(x0?!{,1}),n?) = 
 + S(R(?m2),X(?x2,x3),n)
 - S(R(?m2),X(?x2,x3*(1-x0)),n);
id SAlgCS`i'(R(?m2),X(?x2,x3?),RR(0),XX(1),n?) =  S(R(?m2),X(?x2,x3),n);

id SAlgCS`i'(R(?m),X(?x),RR,XX,n?) = S(R(?m),X(?x),n);
id SAlgCS`i'(R,X,RR(0),XX(x?),n?) = ( 1 - pow((1-x),n) );

*
* convert the SAlgCSS sums back 
*
id SAlgCSS(RR(?m),XX(?x)) = SAlgCSS(RR(?m),XX(?x),R,X);

repeat, id SAlgCSS(RR(?m,m1?,m2?),XX(?x,x1?,x2?),R(?m3),X(?x3)) =
 + SAlgCSS(RR(?m),XX(?x),R(m1,m2,?m3),X(x1,x2,?x3))
 - SAlgCSS(RR(?m),XX(?x),R,X)* S(R(m1,m2,?m3),X(x1,x2,?x3),inf)
 + SAlgCSS(RR(?m,m1),XX(?x,x1),R,X) * S(R(m2,?m3),X(x2,?x3),inf);

id SAlgCSS(RR,XX,R,X) = 1;
id SAlgCSS(RR,XX,R(?m),X(?x)) = S(R(?m),X(?x),0);
id SAlgCSS(RR(m1?),XX(x1?),R(?m),X(?x)) = S(R(m1,?m),X(x1,?x),inf)
                                    - S(R(m1,?m),X(x1,?x),0);
id S(?iii,0) = 0;
id S(R,X,?j) = 1;
id pow(0,x?) = 0;

#endprocedure
*************************************************************************
* Local variables:
* modify-insert-tag: nil
* comment-start: "*"
* tab-width: 2
* End:
*************************************************************************
