#procedure ExpandDen(ii)
****************************************************************************
**
**  Filename:
**            ExpandDen.prc
**
**  Authors: 
**            Sven O. Moch and Peter Uwer
**
**  Purpose: 
**             Expands \keyword{den} function in small parameter \keyword{ep}, 
**             i.e. $1/(a+b\epsilon)$ in terms of $\epsilon$. 
**             If the argument provided to \keyword{ExpandDen}
**             is 0 then denominators with integer values of $a$ are expanded.
**             If the argument is 1, we expand for symbolic $a$.
**
**  Comments:
**
**
**  Revision: 
**            $Id: ExpandDen.prc,v 1.0 2005/08/02 10:09:54 puwer Exp $
**
**            $Log: ExpandDen.prc,v $
**            Revision 1.0  2005/08/02 10:09:54  puwer
**            Initial revision
**
**
****************************************************************************
*
id      num(x?)*den(x?) = 1;
repeat	id num(x1?)*num(x2?) = num(x1*x2);
id      num(x?) = x;
id      ep^n? = acc(ep^n);
repeat	id den(n1?,n2?,?a) = den(n1+n2,?a);

SplitArg,((ep)),den;

id	den(0,x?) = dena(x/ep,x);
id	dena(x?,ep?) = epow(-1)/x;
repeat 	id epow(n1?)*epow(n2?) = epow(n1+n2);
id      epow(n?)*acc(x?) = acc(ep^n*x);

.sort:start ExpandDen (`ii');

#if `ii' = 0

*
* collect denominators which are identical:
*
id	den(n1?number_,ep?) = den(n1,ep,1);
repeat  id den(n1?number_,ep?,l1?)*den(n1?number_,ep?,l2?) = den(n1,ep,l1+l2);

*
* geometric series in ep
*
#do jj = 1,1
Multiply ycounter;
id      acc(x?) = x;
id	ycounter*den(x1?number_,x2?,l1?pos_)*ep^n0? = 
	sum_(jjj,0,`EXPANDEP'-n0,binom_(jjj+l1-1,l1-1)*(-x2/x1)^jjj) / x1^l1*ep^n0
	+ ep^{`EXPANDEP'+1}*order({`EXPANDEP'+1});
id	ycounter = 1;

if(	match(den(x?number_,ep?,l1?pos_)) ) Redefine jj "0";
id      ep^n? = acc(ep^n);
.sort: den(n+m*ep)-(`ii');
#enddo

.sort:ExpandDen (`ii');
#endif

#if `ii' = 1
*
* careful with 1/(j1+a*ep) when j1 = 0
* 
id	den(x1?!number_,x2?) = dena(x1,x2/ep,ep);
id      dena(x1?!number_,x2?,ep?) = 
	den(x1,x2*ep,1)*deltap(x1)+epow(-1)/x2*delta(x1);

id	delta(x?)*delta(x?) = delta(x);
id	deltap(x?)*deltap(x?) = deltap(x);
id	delta(x?)*deltap(x?) = 0;

repeat	id epow(x1?)*epow(x2?) = epow(x1+x2);
id	epow(n?)*acc(x?) = acc(ep^n*x);

repeat, id order({`EXPANDEP'+1})*order({`EXPANDEP'+1}) = order({`EXPANDEP'+1});
*
* collect denominators which are identical:
*
repeat  id den(n1?!number_,ep?,l1?)*den(n1?!number_,ep?,l2?) = den(n1,ep,l1+l2);

*
* geometric series in ep
*
id      acc(x?) = x;
repeat	id den(x1?!number_,x2?,l1?pos_)*ep^n0? = 
	sum_(jjj,0,`EXPANDEP'-n0,binom_(jjj+l1-1,l1-1)*(-x2*den(x1))^jjj)*den(x1)^l1*ep^n0
	+ ep^{`EXPANDEP'+1}*order({`EXPANDEP'+1});
id	ep^n? = acc(ep^n);

repeat	id den(n1?,n2?,?a) = den(n1+n2,?a);
.sort:ExpandDen (`ii');
#endif

id den(x?number_) = 1/x;
repeat, id order({`EXPANDEP'+1})*order({`EXPANDEP'+1}) = order({`EXPANDEP'+1});

#endprocedure
*
*************************************************************************
* Local variables:
* modify-insert-tag: nil
* comment-start: "*"
* tab-width: 2
* End:
*************************************************************************
