#procedure SumIt(i)
****************************************************************************
**
**  Filename:
**            SumIt.prc
**
**  Authors: 
**            Sven O. Moch and Peter Uwer
**
**  Purpose: 
**            Calls the various procedures for summation algorithms. The argument 
**            specifies the index $i$ of the sum to be done.
**
**  Comments:
**            One should be a bit careful with ji+inf -> inf because
**            if ji is a summation parameter that can also go to 
**            infinity we may want to obtain 2*inf in due time. 
**            This regularizes properly for 
**                S(R(1),n*inf) => S(R(1),inf) + ln_(n)
**
**  Revision: 
**            $Id: SumIt.prc,v 1.0 2005/08/02 10:09:54 puwer Exp $
**
**            $Log: SumIt.prc,v $
**            Revision 1.0  2005/08/02 10:09:54  puwer
**            Initial revision
**
**
****************************************************************************
*
if ( count(sum`i',1) );
*
*   go through the thing again in case we obtain a suitable simplification.
*
	Multiply x;
	repeat;
		if ( count(x,1) );
			id x = 1;

*
*   Conjugations
*
			if ( ( count(bino`i',1) == 1 ) && ( count(fac`i',1,invfac`i',1) == 0 ) );  
				id den`i'(j`i') = den`i'(0,j`i');
				id  1/j`i' = den`i'(0,j`i');
				id  den`i'(x1?,j`i') = den`i'(1,x1,j`i');
				repeat id den`i'(n1?,x1?,j`i')*den`i'(n2?,x1?,j`i') = 
					den`i'(n1+n2,x1,j`i');
				if ( count(j`i',1) > 0 );
					repeat id j`i'*S`i'(R(?b),X(?y),n0?,j`i') = 
						+ den`i'(-1,n0,j`i')*S`i'(R(?b),X(?y),n0,j`i')
						- n0*S`i'(R(?b),X(?y),n0,j`i');
					repeat id den`i'(n1?,x1?,j`i')*den`i'(n2?,x1?,j`i') = 
						den`i'(n1+n2,x1,j`i');
					Print "Unsubstituted sums in %t";
					setexitflag;
				else;
					if ( count(den`i',1) == 0 );
						if(match(S`i'(R(?b),X(?y),n0?,j`i')) );
							id S`i'(R(?b),X(?y),n0?,j`i') = 
								+ den`i'(0,n0,j`i')*S`i'(R(?b),X(?y),n0,j`i');
						else;
							Multiply, den`i'(0,0,j`i');
						endif;
					endif;
				endif;
				if ( ( count(S`i',1) == 2 ) && ( match(S`i'(R(?a),X(?x),0,j`i')) )
						&& ( match(S`i'(R(?a),X(?x),n?,-j`i')) ) );
					#call AlgD(`i')
				elseif ( ( count(S`i',1) < 2 ) && ( match(S`i'(R(?a),X(?x),0,j`i')) == 0 )
						&& ( match(S`i'(R(?a),X(?x),n?,-j`i')) ) );
					Multiply, S`i'(R,X,0,j`i');
					#call AlgD(`i')
				elseif ( ( count(S`i',1) = 1 ) && ( match(S`i'(R(?a),X(?x),n?,-j`i')) == 0 ) );
					#call AlgC(`i')
				elseif ( ( count(S`i',1) = 0 ) );
					id sum`i'(j`i',n1?{,0},n?) * bino`i'(n?,0,j`i') * pow`i'(x?,j`i') * 
						den`i'(m?{,0},0,j`i') = pow((1+x),n);
					id sum`i'(j`i',n1?{,1},n?) * bino`i'(n?,0,j`i') * pow`i'(x?,j`i') * 
   					den`i'(m?{,0},0,j`i') = - 1 + pow((1+x),n);
					#call AlgC(`i')
				endif;	
			elseif ( match(S`i'(R(?a),X(?y),x1?,-j`i')) );
*
*   sums involving S(R(...),n-j)
*
				id  den`i'(x1?,j`i') = den`i'(1,x1,j`i');
				repeat id den`i'(n1?,x1?,j`i')*den`i'(n2?,x1?,j`i') = 
					den`i'(n1+n2,x1,j`i');
				if ( ( count(den`i',1) == 0 ) && ( count(j`i',1) == 0 ) );
					if ( count(S`i',1) == 2 );
						#call AlgB0(`i')
					endif;
					id  S`i'(R,X,0,0) = 0;
        	id  S`i'(R,X,?a) = 1;
					if ( count(S`i',1) == 1 );
						id  sum`i'(j`i',n1?,n?)*S`i'(R(?a),X(?y),n?,-j`i')*pow`i'(y1?,j`i') =
							sum`i'(j`i',0,n-n1)*S`i'(R(?a),X(?y),0,j`i')*pow`i'(y1,j`i')*pow(y1,n);
						id  sum`i'(j`i',n1?,n?)*S`i'(R(?a),X(?y),n?,-j`i') =
							sum`i'(j`i',0,n-n1)*S`i'(R(?a),X(?y),0,j`i');
						Multiply x;
					elseif ( ( count(S`i',1) == 0 ) && ( count(sum`i',1) == 1 ) );
						Multiply x;
					endif;
				elseif ( count(den`i',1) == 1 );
					#call AlgB(`i')
				endif;
			elseif ( match(S`i'(R(?a),X(?y),x1?,j`i')) == 1 );
*
*   sums involving a single S(R(...),X(...),a+j)
*
				id  den`i'(x1?,j`i') = den`i'(1,x1,j`i');
				repeat id den`i'(n1?,x1?,j`i')*den`i'(n2?,x1?,j`i') = 
					den`i'(n1+n2,x1,j`i');
				if ( count(den`i',1) == 1 );
					if ( count(pow`i',1) );
						id  sum`i'(j`i',x1?,x2?)*den`i'(n?,x3?,j`i')*pow`i'(y1?,j`i')*
							S`i'(R(?a),X(?y),x3?,j`i') =
							(S(R(n,?a),X(y1,?y),x2+x3)-S(R(n,?a),X(y1,?y),x1+x3-1))*
							pow(y1,-x3);
					else;
						id  sum`i'(j`i',x1?,x2?)*den`i'(n?,x3?,j`i')*
							S`i'(R(?a),X(?y),x3?,j`i') =
							S(R(n,?a),X(1,?y),x2+x3)-S(R(n,?a),X(1,?y),x1+x3-1);
					endif;
				elseif ( ( count(den`i',1) == 0 ) && ( count(j`i',1) == 0 ) );
					if ( count(pow`i',1) );
						id  sum`i'(j`i',x1?{0,1},x2?)*S`i'(R(n1?,?a),X(y1?,?y),0,j`i')*
							pow`i'(y2?,j`i') = 
							den(1-y2)*(-pow(y2,x2+1)*S(R(n1,?a),X(y1,?y),x2)
							+S(R(n1,?a),X(y1*y2,?y),x2));
					else;
						repeat;
							id  sum`i'(j`i',x1?{0,1},x2?)*S`i'(R(n1?pos_,?a),X(y1?,?y),0,j`i') =
								(x2+1)*S(R(n1,?a),X(y1,?y),x2)
								-sum`i'(j`i',1,x2)*S`i'(R(?a),X(?y),0,j`i')*pow`i'(y1,j`i')*delta_(n1-1)
								-S(R(n1-1,?a),X(y1,?y),x2)*theta_(n1-2);
							id  pow`i'(1,j`i') = 1;
							id  sum`i'(j`i',x1?{0,1},x2?)*S`i'(R(n1?,?a),X(y1?,?y),0,j`i')*
								pow`i'(y2?,j`i') = 
								den(1-y2)*(-pow(y2,x2+1)*S(R(n1,?a),X(y1,?y),x2)
								+S(R(n1,?a),X(y1*y2,?y),x2));
						endrepeat;
					endif;
					if ( match(S`i'(R,X,n?,j`i')) );
						id  S`i'(R,X,?a) = 1;
						if ( count(pow`i',1) );
							id  sum`i'(j`i',x1?,x2?)*pow`i'(y1?,j`i') = 
								den(1-y1)*(-pow(y1,x2+1)+pow(y1,x1));
						else;
							id  sum`i'(j`i',x1?,x2?) = x2+1-x1;
						endif;
					endif;
				elseif ( ( count(den`i',1) == 0 ) && ( count(j`i',1) > 0 ) );
					#call SumPosPow(`i')
				endif; 
    	elseif ( count(S`i',1) == 0 );
*
*   sums involving no S at all --> only den, pow or j
*
				id  den`i'(x1?,j`i') = den`i'(1,x1,j`i');
				repeat id den`i'(n1?,x1?,j`i')*den`i'(n2?,x1?,j`i') = 
					den`i'(n1+n2,x1,j`i');
				if ( count(den`i',1) == 1 );
					if ( count(pow`i',1) );
						id  sum`i'(j`i',x1?,x2?)*den`i'(n?,x3?,j`i')*pow`i'(y1?,j`i') =
							S(R(n),X(y1),x2+x3)-S(R(n),X(y1),x1+x3-1);
					else;
						id  sum`i'(j`i',x1?,x2?)*den`i'(n?,x3?,j`i') =
							S(R(n),X(1),x2+x3)-S(R(n),X(1),x1+x3-1);
					endif;
				elseif ( ( count(den`i',1) == 0 ) );
					if ( count(pow`i',1) );
						id  sum`i'(j`i',x1?,x2?)*pow`i'(y1?,j`i')*j`i'^k?pos0_ = xpowsum(k,y1,x2)-xpowsum1(k,y1,x1);
					else;
						id  sum`i'(j`i',x1?,x2?)*j`i'^k?pos0_ = powsum(k,x2)-powsum1(k,x1);
					endif;
				endif;
			endif;
		endif;

		id pow(0,x?) = 0;
		id pow(1,x?) = 1;
		id pow(x?,0) = 1;
		id S(?iii,0) = 0;
		id S(R,X,?j) = 1;
	endrepeat;
endif;

id  den`i'(n?,x1?,x2?) = den(x1,x2)^n;

#endprocedure
*************************************************************************
* Local variables:
* modify-insert-tag: nil
* comment-start: "*"
* tab-width: 2
* End:
*************************************************************************
